/*
  * JBoss, Home of Professional Open Source
  * Copyright 2005, JBoss Inc., and individual contributors as indicated
  * by the @authors tag. See the copyright.txt in the distribution for a
  * full listing of individual contributors.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This software is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this software; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
  */
package org.jboss.test.microcontainer.support;

import org.jboss.aop.joinpoint.Invocation;
import org.jboss.dependency.spi.Controller;

/**
 * A Interceptor with a dependency from an annotation
 * 
 * @author <a href="adrian@jboss.com">Adrian Brock</a>
 * @version $Revision: 63359 $
 */
public class InterceptorWithAnnotationDependency extends AbstractInterceptor
{
   public static final String SRC_CLASS = "class";
   public static final String SRC_METHOD = "method";

   public static SimpleBean intercepted = null;
   
   private static Controller controller;
   
   public void setController(Controller controller)
   {
      InterceptorWithAnnotationDependency.controller = controller;
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      log.debug("InterceptorWithAnnotationDependency: " + invocation);
      TestAnnotationDependency annotation = (TestAnnotationDependency) invocation.resolveClassAnnotation(TestAnnotationDependency.class);
      if (annotation == null)
      {
         annotation = (TestAnnotationDependency) invocation.resolveAnnotation(TestAnnotationDependency.class);
      }

      String data = annotation.data();
      log.debug("Data=" + data);
      intercepted = (SimpleBean) controller.getInstalledContext(data).getTarget();
      return invocation.invokeNext();
   }
}
