package org.jboss.test.microcontainer.support.jndi;

public interface ISimpleBean
{
   public String getProp1();
   public void setProp1(String prop1);
   public int getProp2();
   public void setProp2(int prop2);
   public float getProp3();
   public void setProp3(float prop3);
}
