/*
* JBoss, Home of Professional Open Source.
* Copyright 2006, Red Hat Middleware LLC, and individual contributors
* as indicated by the @author tags. See the copyright.txt file in the
* distribution for a full listing of individual contributors. 
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/ 
package org.jboss.aop.microcontainer.beans;

import java.security.AccessController;
import java.security.PrivilegedAction;

/**
 * CL security actions.
 *
 * @author <a href="kabir.khan@jboss.com">Kabir Khan</a>
 * @version $Revision: 1.1 $
 */
class SecurityActions
{
   private interface GetContextClassLoaderAction
   {
      ClassLoader getContextClassLoader();
      
      GetContextClassLoaderAction PRIVILEGED = new GetContextClassLoaderAction() {

         public ClassLoader getContextClassLoader()
         {
            return AccessController.doPrivileged(new PrivilegedAction<ClassLoader>()
            {
               public ClassLoader run()
               {
                  return Thread.currentThread().getContextClassLoader();
               }
            });
         }
      };

      GetContextClassLoaderAction NOT_PRIVILEGED = new GetContextClassLoaderAction() {

         public ClassLoader getContextClassLoader()
         {
            return Thread.currentThread().getContextClassLoader();
         }
      };
   }
   
   public static ClassLoader getContextClassLoader()
   {
      if (System.getSecurityManager() == null)
      {
         return GetContextClassLoaderAction.NOT_PRIVILEGED.getContextClassLoader();
      }
      else
      {
         return GetContextClassLoaderAction.PRIVILEGED.getContextClassLoader();
      }
   }

   private interface GetClassLoaderAction
   {
      ClassLoader getClassLoader(Class<?> clazz);
      
      GetClassLoaderAction PRIVILEGED = new GetClassLoaderAction(){
      
            public ClassLoader getClassLoader(final Class<?> clazz)
            {
               return AccessController.doPrivileged(new PrivilegedAction<ClassLoader>()
               {
                  public ClassLoader run()
                  {
                     return clazz.getClassLoader();
                  }
               });
            }
      };

      GetClassLoaderAction NOT_PRIVILEGED = new GetClassLoaderAction(){
         
         public ClassLoader getClassLoader(Class<?> clazz)
         {
            return clazz.getClassLoader();
         }
      };
   }
   
   public static ClassLoader getClassLoader(Class<?> clazz)
   {
      if (System.getSecurityManager() == null)
      {
         return GetClassLoaderAction.NOT_PRIVILEGED.getClassLoader(clazz);
      }
      else
      {
         return GetClassLoaderAction.PRIVILEGED.getClassLoader(clazz);
      }
   }
}
